#
# Copyright (C) 2012-2019 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

import hawkey
import unittest


class Package(unittest.TestCase):
    def test_create(self):
        r = hawkey.Repo("fog")
        self.assertIsNotNone(r)
        self.assertRaises(TypeError, hawkey.Repo);
        self.assertRaises(TypeError, hawkey.Repo, 3)
        self.assertRaises(TypeError, hawkey.Repo, rain="pouring")

    def test_cost_assignment(self):
        r = hawkey.Repo("fog")
        r.cost = 300
        self.assertEqual(300, r.cost)

        r2 = hawkey.Repo("blizzard")
        self.assertEqual(1000, r2.cost)
        with self.assertRaises(TypeError):
            r2.cost = '4'

    def test_max_parallel_downloads(self):
        r = hawkey.Repo("fog")
        r.max_parallel_downloads = 10
        self.assertEqual(10, r.max_parallel_downloads)

    def test_max_downloads_per_mirror(self):
        r = hawkey.Repo("fog")
        r.max_downloads_per_mirror = 10
        self.assertEqual(10, r.max_downloads_per_mirror)

    def test_str_assignment(self):
        r = hawkey.Repo('fog')
        with self.assertRaises(TypeError):
            r.repomd_fn = 3
        r.repomd_fn = 'rain'
        self.assertEqual(r.repomd_fn, 'rain')




#
# Copyright (C) 2012-2013 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import
from . import base
from copy import deepcopy

import hawkey

class GoalTest(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        self.sack.load_test_repo("main", "main.repo")

    def test_actions(self):
        sltr = hawkey.Selector(self.sack).set(name="walrus")
        goal = hawkey.Goal(self.sack)

        self.assertEqual(0, goal.actions & (hawkey.ERASE | hawkey.DISTUPGRADE |
            hawkey.DISTUPGRADE_ALL | hawkey.DOWNGRADE | hawkey.INSTALL | hawkey.UPGRADE |
            hawkey.UPGRADE_ALL))
        goal.upgrade(select=sltr)
        self.assertEqual(hawkey.UPGRADE, goal.actions)

    def test_clone(self):
        pkg = base.by_name(self.sack, "penny-lib")
        goal = hawkey.Goal(self.sack)
        goal.erase(pkg)
        self.assertFalse(goal.run())

        goal2 = deepcopy(goal)
        self.assertTrue(goal2.run(allow_uninstall=True))
        self.assertEqual(len(goal2.list_erasures()), 2)

        goal3 = deepcopy(goal)
        goal3.add_protected(hawkey.Query(self.sack).filter(name="flying"))
        self.assertFalse(goal3.run(allow_uninstall=True))

    def test_list_err(self):
        goal = hawkey.Goal(self.sack)
        self.assertRaises(hawkey.ValueException, goal.list_installs)

    def test_empty_selector(self):
        sltr = hawkey.Selector(self.sack)
        goal = hawkey.Goal(self.sack)
        # does not raise ValueException
        goal.install(select=sltr)
        goal.run()
        self.assertEqual(goal.list_installs(), [])

    def test_install_selector(self):
        sltr = hawkey.Selector(self.sack).set(name="walrus")
        # without checking versioning, the update is accepted:
        self.assertIsNone(hawkey.Goal(self.sack).upgrade(select=sltr))

    def test_install_selector_weak(self):
        sltr = hawkey.Selector(self.sack).set(name='hello')
        goal = hawkey.Goal(self.sack)
        goal.install(select=sltr, optional=True)
        self.assertTrue(goal.run())

    def test_install_selector_err(self):
        sltr = hawkey.Selector(self.sack)
        self.assertRaises(hawkey.ValueException, sltr.set, undefined="eapoe")

    def test_reinstall(self):
        inst = base.by_name_repo(self.sack, "fool", hawkey.SYSTEM_REPO_NAME)
        avail = base.by_name_repo(self.sack, "fool", "main")
        goal = hawkey.Goal(self.sack)
        goal.install(avail)
        self.assertTrue(goal.run())
        self.assertLength(goal.list_erasures(), 0)
        self.assertLength(goal.list_installs(), 0)
        self.assertLength(goal.list_reinstalls(), 1)
        reinstall = goal.list_reinstalls()[0]
        obsoleted = goal.obsoleted_by_package(reinstall)
        self.assertItemsEqual(list(map(str, obsoleted)), ("fool-1-3.noarch", ))

    def test_req(self):
        goal = hawkey.Goal(self.sack)
        self.assertEqual(goal.req_length(), 0)
        self.assertFalse(goal.req_has_erase())
        sltr = hawkey.Selector(self.sack).set(name="jay")
        goal.erase(select=sltr)
        self.assertEqual(goal.req_length(), 1)
        self.assertTrue(goal.req_has_erase())

        goal = hawkey.Goal(self.sack)
        goal.upgrade(select=sltr)
        self.assertFalse(goal.req_has_erase())

        goal = hawkey.Goal(self.sack)
        pkg = hawkey.Query(self.sack).filter(name="dog")[0]
        goal.erase(pkg, clean_deps=True)
        self.assertTrue(goal.req_has_erase())


class GoalRunAll(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        self.sack.load_test_repo("greedy", "greedy.repo")
        self.goal = hawkey.Goal(self.sack)

    def test_goal_install_weak_deps(self):
        pkg_b = base.by_name(self.sack, "B")
        self.goal.install(pkg_b)
        self.assertTrue(self.goal.run())
        installs = self.goal.list_installs()
        self.assertEqual(len(installs), 2)
        expected_installs = ("B-1-0.noarch", "C-1-0.noarch")
        self.assertItemsEqual(list(map(str, installs)), expected_installs)
        goal2 = deepcopy(self.goal)
        self.assertTrue(goal2.run(ignore_weak_deps=True))
        installs2 = goal2.list_installs()
        self.assertEqual(len(goal2.list_installs()), 1)
        self.assertEqual(str(installs2[0]), "B-1-0.noarch")



#!/usr/bin/python


import os
import argparse
import subprocess
import tempfile

import gi
gi.require_version('Modulemd', '1.0')
from gi.repository import Modulemd


def get_parser():
    """
    Construct argument parser.

    :returns: ArgumentParser object with arguments set up.
    :rtype: argparse.ArgumentParser
    """
    parser = argparse.ArgumentParser(
        description="Scan directory for modulemd yaml files and inject them into repodata.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "path",
        metavar="directory_to_index",
    )
    return parser


def index_modulemd_files(repo_path):
    result = []
    for fn in sorted(os.listdir(repo_path)):
        if not fn.endswith(".yaml"):
            continue
        yaml_path = os.path.join(repo_path, fn)
        mmd = Modulemd.objects_from_file_ext(yaml_path)
        result.append(mmd[0][0])
    return result


def modify_repo(repo_path, modules):
    tmp = tempfile.mkdtemp()
    path = os.path.join(tmp, "modules.yaml")
    for module in modules:
        Modulemd.dump(modules, path)
    subprocess.check_call(["modifyrepo_c", "--mdtype=modules", path,
                           os.path.join(repo_path, "repodata")])
    os.unlink(path)
    os.rmdir(tmp)


if __name__ == "__main__":
    parser = get_parser()
    args = parser.parse_args()

    modules = index_modulemd_files(args.path)
    modify_repo(args.path, modules)



#
# Copyright (C) 2012-2013 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import

from . import base
import hawkey
import unittest

class Util(unittest.TestCase):
    def test_chksum_name(self):
        name = hawkey.chksum_name(hawkey.CHKSUM_SHA256)
        self.assertEqual(name, "sha256")

    def test_chksum_type(self):
        t = hawkey.chksum_type("SHA1")
        self.assertEqual(t, hawkey.CHKSUM_SHA1)
        self.assertRaises(ValueError, hawkey.chksum_type, "maID")

    def test_split_nevra(self):
        self.assertRaises(hawkey.ValueException, hawkey.split_nevra, "no.go")
        self.assertRaises(hawkey.ValueException, hawkey.split_nevra, "")
        self.assertRaises(hawkey.ValueException, hawkey.split_nevra, "eyes-8:-4.fc18.x86_64")
        self.assertRaises(hawkey.ValueException, hawkey.split_nevra, "eyes-4.fc18.x86_64")
        self.assertRaises(hawkey.ValueException, hawkey.split_nevra, "eyes--4.fc18.x86_64")
        self.assertRaises(hawkey.ValueException, hawkey.split_nevra, "eyes-8:1.2.3-4.")

        nevra = hawkey.split_nevra("eyes-8:1.2.3-4.fc18.x86_64")
        self.assertEqual(nevra.name, "eyes")
        self.assertEqual(nevra.epoch, 8)
        self.assertEqual(nevra.version, "1.2.3")
        self.assertEqual(nevra.release, "4.fc18")
        self.assertEqual(nevra.arch, "x86_64")

class UtilWithSack(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()

    def test_nevra_to_query(self):
        nevra = hawkey.split_nevra("baby-6:5.0-11.x86_64")
        q = nevra.to_query(self.sack)
        self.assertLength(q, 1)
        pkg = str(q[0])
        self.assertEqual(pkg, "baby-6:5.0-11.x86_64")



#
# Copyright (C) 2014 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

"""Tests of the _hawkey.AdvisoryRef class."""

from __future__ import absolute_import
from __future__ import unicode_literals
from . import base

import hawkey
import itertools

def find_reference(sack, url):
    """Find an advisory reference with given URL."""
    # The function is needed because references cannot be retrieved directly.
    advisories_iterable = itertools.chain(
        (pkg.get_advisories(hawkey.LT) for pkg in hawkey.Query(sack)),
        (pkg.get_advisories(hawkey.GT | hawkey.EQ) for pkg in hawkey.Query(sack)))
    advisories = itertools.chain.from_iterable(advisories_iterable)
    references_iterable = (advisory.references for advisory in advisories)
    for reference in itertools.chain.from_iterable(references_iterable):
        if reference.url == url:
            return reference

class Test(base.TestCase):
    """Test case consisting of one random advisory reference."""

    def setUp(self):
        """Prepare the test fixture."""
        sack = base.TestSack(repo_dir=self.repo_dir)
        sack.load_repo(load_updateinfo=True)
        self.reference = find_reference(
            sack, 'https://bugzilla.redhat.com/show_bug.cgi?id=472090')

    def test_type(self):
        self.assertEqual(self.reference.type, hawkey.REFERENCE_BUGZILLA)

    def test_id(self):
        self.assertEqual(self.reference.id, '472090')

    def test_title(self):
        self.assertEqual(self.reference.title,
                         '/etc/init.d/clvmd points to /usr/sbin for LVM tools')

    def test_url(self):
        self.assertEqual(self.reference.url,
                         'https://bugzilla.redhat.com/show_bug.cgi?id=472090')



#!/usr/bin/python


import os
import json
import hashlib

import gi

gi.require_version('Modulemd', '1.0')
from gi.repository import Modulemd

MODULES_DIR = os.path.join(os.path.dirname(__file__), "..", "modules")
SPECS_DIR = os.path.join(os.path.dirname(__file__), "..", "specs")


def parse_module_id(module_id):
    return module_id.rsplit("-", 2)


for module_id in os.listdir(MODULES_DIR):
    if module_id.startswith("_"):
        continue

    name, stream, version = parse_module_id(module_id)

    profiles_file = os.path.join(SPECS_DIR, module_id, "profiles.json")
    if os.path.isfile(profiles_file):
        with open(profiles_file, "r") as f:
            profiles = json.load(f)
    else:
        profiles = {}

    for arch in os.listdir(os.path.join(MODULES_DIR, module_id)):
        if arch == "noarch":
            continue

        module_dir = os.path.join(MODULES_DIR, module_id, arch)
        rpms = [i for i in os.listdir(module_dir) if i.endswith(".rpm")]

        noarch_module_dir = os.path.join(MODULES_DIR, module_id, "noarch")
        if os.path.isdir(noarch_module_dir):
            noarch_rpms = [i for i in os.listdir(noarch_module_dir) if i.endswith(".rpm")]
        else:
            noarch_rpms = []

        rpms = sorted(set(rpms) | set(noarch_rpms))

        # HACK: force epoch to make test data compatible with libmodulemd >= 1.4.0
        rpms_with_epoch = []
        for i in rpms:
            n, v, ra = i.rsplit("-", 2)
            nevra = "%s-0:%s-%s" % (n, v, ra)
            rpms_with_epoch.append(nevra)
        rpms = rpms_with_epoch

        mmd = Modulemd.Module()
        mmd.set_mdversion(int(2))
        mmd.set_name(name)
        mmd.set_stream(stream)
        mmd.set_version(int(version))
        # context is set later, computed from runtime deps
        mmd.set_arch(arch)
        sset = Modulemd.SimpleSet()
        sset.add("LGPLv2")
        mmd.set_module_licenses(sset)
        mmd.set_summary("Fake module")
        mmd.set_description(mmd.get_summary())
        artifacts = Modulemd.SimpleSet()
        for rpm in rpms:
            #mmd.add_module_component(rpm.rsplit("-", 2)[0], "")
            artifacts.add(rpm[:-4])
        mmd.set_rpm_artifacts(artifacts)
        for profile_name in profiles:
            profile = Modulemd.Profile()
            profile.set_name(profile_name)
            profile_rpms = Modulemd.SimpleSet()
            profile_rpms.set(profiles[profile_name]["rpms"])
            profile.set_rpms(profile_rpms)
            mmd.add_profile(profile)

        if name == "httpd":
            dependencies = Modulemd.Dependencies()
            if stream == "2.4":
                dependencies.add_requires_single("base-runtime", "f26")
            elif stream == "2.2":
                dependencies.add_requires("base-runtime", [])
            mmd.add_dependencies(dependencies)

        # iterate through all deps and create context hash in a repeatable manner
        context_hash = hashlib.sha256()
        for dependencies in mmd.peek_dependencies():
            for dep_name, dep_streams in dependencies.peek_requires().items():
                if dep_streams:
                    for dep_stream in dep_streams.get():
                        context_hash.update("%s:%s" % (dep_name, dep_stream))
                else:
                    context_hash.update(dep_name)
        mmd.set_context(context_hash.hexdigest()[:8])

        Modulemd.dump([mmd], os.path.join(module_dir, "%s.%s.yaml" % (module_id, arch)))



#
# Copyright (C) 2012-2019 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from sys import version_info as python_version

import libdnf
import hawkey
import hawkey.test
import os
import tempfile
import unittest

cachedir = None
# inititialize the dir once and share it for all sacks within a single run
if cachedir is None:
    cachedir = tempfile.mkdtemp(dir=os.path.dirname(hawkey.test.UNITTEST_DIR),
                                prefix='pyhawkey')

class TestCase(unittest.TestCase):
    repo_dir = os.path.normpath(os.path.join(__file__, "../../../../../data/tests/hawkey/"))

    def assertLength(self, collection, length):
        return self.assertEqual(len(collection), length)

    def assertItemsEqual(self, item1, item2):
        # assertItemsEqual renamed in Python3
        if python_version.major < 3:
            super(TestCase, self).assertItemsEqual(item1, item2)
        else:
            super(TestCase, self).assertCountEqual(item1, item2)

skip = unittest.skip

class TestSack(hawkey.test.TestSackMixin, hawkey.Sack):
    def __init__(self, repo_dir, PackageClass=None, package_userdata=None,
                 make_cache_dir=True, arch=None, all_arch=False):
        hawkey.test.TestSackMixin.__init__(self, repo_dir)
        hawkey.Sack.__init__(self,
                             cachedir=cachedir,
                             arch=arch if arch is not None else hawkey.test.FIXED_ARCH,
                             pkgcls=PackageClass,
                             pkginitval=package_userdata,
                             make_cache_dir=make_cache_dir,
                             all_arch=all_arch,
                             )

    def load_repo(self, **kwargs):
        d = os.path.join(self.repo_dir, 'yum/')
        self._conf = libdnf.conf.ConfigMain()
        repo_conf = libdnf.conf.ConfigRepo(self._conf)
        self._conf.cachedir().set(libdnf.conf.Option.Priority_REPOCONFIG, self.cache_dir)
        repo_conf.baseurl().set(libdnf.conf.Option.Priority_REPOCONFIG, 'file://%s' % d)
        repo_conf.this.disown()  # _repo will be the owner of _config
        repo = libdnf.repo.Repo("messerk", repo_conf)
        repo.load()
        super(TestSack, self).load_repo(repo, **kwargs)

    # Loading using hawkey.Repo
    def load_repo_hawkey_Repo(self, **kwargs):
        d = os.path.join(self.repo_dir, hawkey.test.YUM_DIR_SUFFIX)
        repo = hawkey.test.glob_for_repofiles(self, "messerk", d)
        super(TestSack, self).load_repo(repo, **kwargs)

def by_name(sack, name):
    return hawkey.Query(sack).filter(name=name)[0]

def by_name_repo(sack, name, repo):
    return hawkey.Query(sack).filter(name=name, reponame=repo)[0]



#
# Copyright (C) 2012-2013 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import
from . import base

import hawkey

class Reldep(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()

    def test_basic(self):
        flying = base.by_name(self.sack, "flying")
        requires = flying.requires
        self.assertLength(requires, 1)
        reldep = requires[0]
        self.assertEqual(str(reldep), "P-lib >= 3")

    def test_custom_creation(self):
        reldep_str = "P-lib >= 3"
        reldep = hawkey.Reldep(self.sack, reldep_str)
        self.assertEqual(reldep_str, str(reldep))
        reldep_str = "lane = 4"
        reldep = hawkey.Reldep(self.sack, reldep_str)
        self.assertEqual(reldep_str, str(reldep))
        reldep_str = "(foo if bar)"
        reldep = hawkey.Reldep(self.sack, reldep_str)
        self.assertEqual(reldep_str, str(reldep))
        reldep_str = "font(:lang=en)"
        reldep = hawkey.Reldep(self.sack, reldep_str)
        self.assertEqual(reldep_str, str(reldep))

    def test_custom_creation_fail(self):
        reldep_str = "P-lib >="
        self.assertRaises(hawkey.ValueException, hawkey.Reldep, self.sack,
                          reldep_str)
        reldep_str = "(foo i bar)"
        self.assertRaises(hawkey.ValueException, hawkey.Reldep, self.sack,
                          reldep_str)

    def test_custom_querying(self):
        reldep = hawkey.Reldep(self.sack, "P-lib = 3-3")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 1)
        # '==' operator is deprecated and the support will be dropped in future
        # versions (see bug 1847946)
        reldep = hawkey.Reldep(self.sack, "P-lib == 3-3")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 1)
        reldep = hawkey.Reldep(self.sack, "P-lib >= 3")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 1)
        reldep = hawkey.Reldep(self.sack, "(foo or P-lib)")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 1)
        reldep = hawkey.Reldep(self.sack, "P-lib < 3-3")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 0)
        reldep = hawkey.Reldep(self.sack, u"\u0159 >= 3")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 0)
        reldep = hawkey.Reldep(self.sack, "foo >= 1.0-1.fc20")
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 0)

    def test_query_name_only(self):
        reldep_str = "P-lib"
        reldep = hawkey.Reldep(self.sack, reldep_str)
        self.assertEqual(reldep_str, str(reldep))
        q = hawkey.Query(self.sack).filter(provides=reldep)
        self.assertLength(q, 1)
        self.assertEqual(str(q[0]), "penny-lib-4-1.x86_64")

    def test_not_crash(self):
        self.assertRaises(ValueError, hawkey.Reldep)



#
# Copyright (C) 2012-2013 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import
from . import base
from hawkey import NEVRA
from sys import version_info as python_version

import hawkey

INP_FOF="four-of-fish-8:3.6.9-11.fc100.x86_64"
INP_FOF_NOEPOCH="four-of-fish-3.6.9-11.fc100.x86_64"
INP_FOF_NEV="four-of-fish-8:3.6.9"
INP_FOF_NA="four-of-fish-3.6.9.i686"

class SubjectTest(base.TestCase):
    def test_nevra(self):
        subj = hawkey.Subject(INP_FOF)
        result = subj.get_nevra_possibilities(forms=hawkey.FORM_NEVRA)
        self.assertLength(result, 1)
        self.assertEqual(result[0], NEVRA(name='four-of-fish', epoch=8,
                                                 version='3.6.9',
                                                 release='11.fc100',
                                                 arch='x86_64'))

        subj = hawkey.Subject(INP_FOF_NOEPOCH)
        result = subj.get_nevra_possibilities(forms=hawkey.FORM_NEVRA)
        self.assertEqual(result[0], NEVRA(name='four-of-fish', epoch=None,
                                                 version='3.6.9',
                                                 release='11.fc100',
                                                 arch='x86_64'))

    def test_nevr(self):
        subj = hawkey.Subject(INP_FOF)
        expect = NEVRA(name='four-of-fish', epoch=8, version='3.6.9',
                              release='11.fc100.x86_64', arch=None)
        self.assertEqual(subj.get_nevra_possibilities(forms=hawkey.FORM_NEVR)[0],
                         expect)

    def test_nevr_fail(self):
        subj = hawkey.Subject("four-of")
        self.assertLength(subj.get_nevra_possibilities(forms=hawkey.FORM_NEVR),
                          0)

    def test_nev(self):
        subj = hawkey.Subject(INP_FOF_NEV)
        nevra_possibilities = subj.get_nevra_possibilities(forms=hawkey.FORM_NEV)
        self.assertLength(nevra_possibilities, 1)
        self.assertEqual(nevra_possibilities[0],
                         NEVRA(name='four-of-fish', epoch=8,
                                      version='3.6.9', release=None, arch=None))

    def test_na(self):
        subj = hawkey.Subject(INP_FOF_NA)
        nevra_possibilities = subj.get_nevra_possibilities(forms=hawkey.FORM_NA)
        self.assertLength(nevra_possibilities, 1)
        self.assertEqual(nevra_possibilities[0],
                         NEVRA(name='four-of-fish-3.6.9', epoch=None,
                                      version=None, release=None, arch='i686'))

    def test_custom_list(self):
        subj = hawkey.Subject(INP_FOF)
        result = subj.get_nevra_possibilities(forms=[hawkey.FORM_NEVRA,
                                              hawkey.FORM_NEVR])
        self.assertLength(result, 2)
        self.assertEqual(result[0], NEVRA(name='four-of-fish', epoch=8,
                                                 version='3.6.9',
                                                 release='11.fc100',
                                                 arch='x86_64'))
        self.assertEqual(result[1], NEVRA(name='four-of-fish', epoch=8, version='3.6.9',
                              release='11.fc100.x86_64', arch=None))

    def test_combined(self):
        """ Test we get all the possible NEVRA parses. """
        subj = hawkey.Subject(INP_FOF)
        # the epoch in INP_FOF nicely limits the nevra_possibilities:
        expect = (
            NEVRA(name='four-of-fish', epoch=8, version='3.6.9', release='11.fc100', arch='x86_64'),
            NEVRA(name='four-of-fish', epoch=8, version='3.6.9', release='11.fc100.x86_64',
                  arch=None)
        )
        result = subj.get_nevra_possibilities()
        self.assertEqual(len(result), len(expect))
        for idx in range(0, len(expect)):
            self.assertEqual(expect[idx], result[idx])

        subj = hawkey.Subject(INP_FOF_NOEPOCH)
        expect = (
            NEVRA(name='four-of-fish', epoch=None, version='3.6.9',
                  release='11.fc100', arch='x86_64'),
            NEVRA(name='four-of-fish-3.6.9-11.fc100', epoch=None, version=None,
                  release=None, arch='x86_64'),
            NEVRA(name='four-of-fish-3.6.9-11.fc100.x86_64', epoch=None, version=None,
                  release=None, arch=None),
            NEVRA(name='four-of-fish', epoch=None, version='3.6.9',
                  release='11.fc100.x86_64', arch=None),
            NEVRA(name='four-of-fish-3.6.9', epoch=None, version='11.fc100.x86_64',
                  release=None, arch=None)
        )
        result = subj.get_nevra_possibilities()
        self.assertEqual(len(result), len(expect))
        for idx in range(0, len(expect)):
            self.assertEqual(expect[idx], result[idx])




#
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import
from . import base

import hawkey
import sys
import unittest

class TestQuery(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()

    def test_sanity(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__eq="flying")
        self.assertEqual(q.count(), 1)

    def test_creation_empty_sack(self):
        s = hawkey.Sack(make_cache_dir=True)
        q = hawkey.Query(s)

    def test_exception(self):
        q = hawkey.Query(self.sack)
        self.assertRaises(hawkey.ValueException, q.filter, flying__eq="name")
        self.assertRaises(hawkey.ValueException, q.filter, flying="name")

    def test_unicode(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__eq=u"flying")
        self.assertEqual(q.count(), 1)

        q = hawkey.Query(self.sack)
        q.filterm(name__eq=[u"flying", "penny"])
        self.assertEqual(q.count(), 2)

    def test_count(self):
        q = hawkey.Query(self.sack).filter(name=["flying", "penny"])
        self.assertEqual(len(q), 2)
        self.assertEqual(len(q), q.count())
        self.assertTrue(q)

        q = hawkey.Query(self.sack).filter(name="naturalE")
        self.assertFalse(q)
        self.assertEqual(len(q.run()), 0)

    def test_kwargs_check(self):
        q = hawkey.Query(self.sack)
        self.assertRaises(hawkey.ValueException, q.filter, name="flying", upgrades="maracas")

    def test_kwargs(self):
        q = hawkey.Query(self.sack)
        # test combining several criteria
        q.filterm(name__glob="*enny*", summary__substr="eyes")
        self.assertEqual(q.count(), 1)
        # test shortcutting for equality comparison type
        q = hawkey.Query(self.sack)
        q.filterm(name="flying")
        self.assertEqual(q.count(), 1)
        # test flags parsing
        q = hawkey.Query(self.sack).filter(name="FLYING")
        self.assertEqual(q.count(), 0)
        q = hawkey.Query(self.sack).filter(hawkey.ICASE, name="FLYING")
        self.assertEqual(q.count(), 1)

    def test_in(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__substr=["ool", "enny-li"])
        self.assertEqual(q.count(), 2)

    def test_in_set(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__substr=set(["ool", "enny-li"]))
        self.assertEqual(q.count(), 2)

    def test_iteration(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__substr=["penny"])
        self.assertEqual(q.count(), 2)
        self.assertNotEqual(q[0], q[1])

    def test_clone(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__substr=["penny"])
        q_clone = hawkey.Query(query=q)
        del q

        self.assertEqual(q_clone.count(), 2)
        self.assertNotEqual(q_clone[0], q_clone[1])

    def test_clone_with_evaluation(self):
        q = hawkey.Query(self.sack)
        q.filterm(name__substr="penny")
        q.run()
        q_clone = hawkey.Query(query=q)
        del q
        self.assertTrue(q_clone.evaluated)
        self.assertLength(q_clone.run(), 2)

    def test_immutability(self):
        q = hawkey.Query(self.sack).filter(name="jay")
        q2 = q.filter(evr="5.0-0")
        self.assertEqual(q.count(), 2)
        self.assertEqual(q2.count(), 1)

    def test_copy_lazyness(self):
        q = hawkey.Query(self.sack).filter(name="jay")
        self.assertLength(q.run(), 2)
        q2 = q.filter(evr="5.0-0")
        self.assertLength(q2.run(), 1)

    def test_empty(self):
        q = hawkey.Query(self.sack).filter(empty=True)
        self.assertLength(q, 0)
        q = hawkey.Query(self.sack)
        self.assertRaises(hawkey.ValueException, q.filter, empty=False)

    def test_epoch(self):
        q = hawkey.Query(self.sack).filter(epoch__gt=4)
        self.assertEqual(len(q), 1)
        self.assertEqual(q[0].epoch, 6)

    def test_version(self):
        q = hawkey.Query(self.sack).filter(version__gte="5.0")
        self.assertEqual(len(q), 3)
        q = hawkey.Query(self.sack).filter(version__glob="1.2*")
        self.assertLength(q, 2)

    def test_package_in(self):
        pkgs = list(hawkey.Query(self.sack).filter(name=["flying", "penny"]))
        q = hawkey.Query(self.sack).filter(pkg=pkgs)
        self.assertEqual(len(q), 2)
        q2 = q.filter(version__gt="3")
        self.assertEqual(len(q2), 1)

    def test_nevra_match(self):
        query = hawkey.Query(self.sack).filter(nevra__glob="*lib*64")
        self.assertEqual(len(query), 1)
        self.assertEqual(str(query[0]), "penny-lib-4-1.x86_64")

    def test_nevra_strict_match(self):
        query = hawkey.Query(self.sack).filter(nevra_strict="penny-lib-4-1.x86_64")
        self.assertEqual(len(query), 1)
        self.assertEqual(str(query[0]), "penny-lib-4-1.x86_64")

    def test_nevra_strict_list_match(self):
        query = hawkey.Query(self.sack).filter(nevra_strict=["penny-lib-4-1.x86_64"])
        self.assertEqual(len(query), 1)
        self.assertEqual(str(query[0]), "penny-lib-4-1.x86_64")

    def test_nevra_strict_epoch0_match(self):
        query = hawkey.Query(self.sack).filter(nevra_strict="penny-lib-0:4-1.x86_64")
        self.assertEqual(len(query), 1)
        self.assertEqual(str(query[0]), "penny-lib-4-1.x86_64")

    def test_repeated(self):
        q = hawkey.Query(self.sack).filter(name="jay")
        q.filterm(latest_per_arch=True)
        self.assertEqual(len(q), 1)

    def test_latest(self):
        q = hawkey.Query(self.sack).filter(name="pilchard")
        q.filterm(latest_per_arch=True)
        self.assertEqual(len(q), 2)
        q.filterm(latest=True)
        self.assertEqual(len(q), 2)

    def test_reldep(self):
        flying = base.by_name(self.sack, "flying")
        requires = flying.requires
        q = hawkey.Query(self.sack).filter(provides=requires[0])
        self.assertEqual(len(q), 1)
        self.assertEqual(str(q[0]), "penny-lib-4-1.x86_64")
        self.assertRaises(hawkey.QueryException, q.filter, provides__gt=requires[0])

        q = hawkey.Query(self.sack).filter(provides="thisisnotgoingtoexist")
        self.assertLength(q.run(), 0)

    def test_reldep_list(self):
        self.sack.load_test_repo("updates", "updates.repo")
        fool = base.by_name_repo(self.sack, "fool", "updates")
        q = hawkey.Query(self.sack).filter(provides=fool.obsoletes)
        self.assertEqual(str(q.run()[0]), "penny-4-1.noarch")

    def test_disabled_repo(self):
        self.sack.disable_repo(hawkey.SYSTEM_REPO_NAME)
        q = hawkey.Query(self.sack).filter(name="jay")
        self.assertLength(q.run(), 0)
        self.sack.enable_repo(hawkey.SYSTEM_REPO_NAME)
        q = hawkey.Query(self.sack).filter(name="jay")
        self.assertLength(q.run(), 2)

    def test_multiple_flags(self):
        q = hawkey.Query(self.sack).filter(name__glob__not=["p*", "j*"])
        self.assertItemsEqual(list(map(lambda p: p.name, q.run())),
                              ["baby", "bloop", "dog", "flying", "fool", "gun", "tour"])

    def test_apply(self):
        q = hawkey.Query(self.sack).filter(name__glob__not="p*").apply()
        res = q.filter(name__glob__not="j*").run()
        self.assertItemsEqual(list(map(lambda p: p.name, res)),
                              ["baby", "bloop", "dog", "flying", "fool", "gun", "tour"])

    def test_provides_glob_should_work(self):
        q1 = hawkey.Query(self.sack).filter(provides__glob="penny*")
        self.assertLength(q1, 2)
        q2 = hawkey.Query(self.sack).filter(provides__glob="*")
        q3 = hawkey.Query(self.sack).filter()
        self.assertEqual(len(q2), len(q3))
        q4 = hawkey.Query(self.sack).filter(provides__glob="P-l*b >= 3")
        self.assertLength(q4, 1)

    def test_provides_glob_should_not_work(self):
        q = hawkey.Query(self.sack).filter(provides="*")
        self.assertLength(q, 0)
        q = hawkey.Query(self.sack).filter(provides__glob="nomatch*")
        self.assertLength(q, 0)

    def test_provides_list_of_strings(self):
        q1 = hawkey.Query(self.sack).filter(provides=["penny", "P-lib"])
        q2 = hawkey.Query(self.sack).filter(provides__glob=["penny", "P-lib"])
        self.assertEqual(len(q1), len(q2))
        q3 = hawkey.Query(self.sack).filter(provides__glob=["penny*", "P-lib*"])
        self.assertEqual(len(q1), len(q3))
        q4 = hawkey.Query(self.sack).filter(provides__glob=["nomatch*", "P-lib*"])
        q5 = hawkey.Query(self.sack).filter(provides__glob="P-lib*")
        self.assertEqual(len(q4), len(q5))
        q6 = hawkey.Query(self.sack).filter(provides=["nomatch", "P-lib"])
        q7 = hawkey.Query(self.sack).filter(provides="P-lib")
        self.assertEqual(len(q6), len(q7))
        q8 = hawkey.Query(self.sack).filter(provides=[])
        self.assertLength(q8, 0)
        q9 = hawkey.Query(self.sack).filter(provides__glob=[])
        self.assertLength(q9, 0)

    def test_requires_list_of_strings(self):
        q1 = hawkey.Query(self.sack).filter(requires=["penny", "P-lib"])
        q2 = hawkey.Query(self.sack).filter(requires__glob=["penny", "P-lib"])
        self.assertEqual(len(q1), len(q2))
        q3 = hawkey.Query(self.sack).filter(requires__glob=["penny*", "P-lib*"])
        self.assertEqual(len(q1), len(q3))
        q4 = hawkey.Query(self.sack).filter(requires__glob=["nomatch*", "P-lib*"])
        q5 = hawkey.Query(self.sack).filter(requires__glob="P-lib*")
        self.assertEqual(len(q4), len(q5))
        q6 = hawkey.Query(self.sack).filter(requires=["nomatch", "P-lib"])
        q7 = hawkey.Query(self.sack).filter(requires="P-lib")
        self.assertEqual(len(q6), len(q7))
        q1 = hawkey.Query(self.sack).filter(requires__glob=["*bin/away"])
        self.assertLength(q1, 1)


class TestQueryAllRepos(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        self.sack.load_test_repo("main", "main.repo")
        self.sack.load_test_repo("updates", "updates.repo")

    def test_requires(self):
        reldep = hawkey.Reldep(self.sack, "semolina = 2")
        q = hawkey.Query(self.sack).filter(requires=reldep)
        self.assertItemsEqual(list(map(str, q.run())),
                              ["walrus-2-5.noarch", "walrus-2-6.noarch"])

        reldep = hawkey.Reldep(self.sack, "semolina > 1.0")
        q = hawkey.Query(self.sack).filter(requires=reldep)
        self.assertItemsEqual(list(map(str, q.run())),
                              ["walrus-2-5.noarch", "walrus-2-6.noarch"])

    def test_obsoletes(self):
        reldep = hawkey.Reldep(self.sack, "penny < 4-0")
        q = hawkey.Query(self.sack).filter(obsoletes=reldep)
        self.assertItemsEqual(list(map(str, q.run())), ["fool-1-5.noarch"])

    def test_downgradable(self):
        query = hawkey.Query(self.sack).filter(downgradable=True)
        self.assertEqual({str(pkg) for pkg in query},
                         {"baby-6:5.0-11.x86_64", "jay-5.0-0.x86_64"})

    def test_rco_glob(self):
        q1 = hawkey.Query(self.sack).filter(requires__glob="*")
        self.assertLength(q1, 12)
        q2 = hawkey.Query(self.sack).filter(requires="*")
        self.assertLength(q2, 0)
        q3 = hawkey.Query(self.sack).filter(conflicts__glob="cu*")
        self.assertLength(q3, 3)
        q1 = hawkey.Query(self.sack).filter(obsoletes__glob="*")
        self.assertLength(q1, 1)
        q1 = hawkey.Query(self.sack).filter(recommends__glob="*")
        self.assertLength(q1, 1)
        q1 = hawkey.Query(self.sack).filter(enhances__glob="*")
        self.assertLength(q1, 1)
        q1 = hawkey.Query(self.sack).filter(suggests__glob="*")
        self.assertLength(q1, 1)
        q1 = hawkey.Query(self.sack).filter(supplements__glob="*")
        self.assertLength(q1, 1)
        q1 = hawkey.Query(self.sack).filter(requires__glob="*bin/away")
        self.assertLength(q1, 1)


class TestQueryUpdates(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        self.sack.load_test_repo("updates", "updates.repo")

    def test_upgradable(self):
        query = hawkey.Query(self.sack).filter(upgradable=True)
        self.assertEqual({str(pkg) for pkg in query},
                         {"bloop-1.0-1.noarch", "dog-1-1.x86_64",
                          "flying-2-9.noarch", "fool-1-3.noarch",
                          "pilchard-1.2.3-1.i686", "pilchard-1.2.3-1.x86_64"})

    def test_updates_noarch(self):
        q = hawkey.Query(self.sack)
        q.filterm(name="flying", upgrades=1)
        self.assertEqual(q.count(), 3)

    def test_updates_arch(self):
        q = hawkey.Query(self.sack)
        pilchard = q.filter(name="dog", upgrades=True)
        self.assertItemsEqual(list(map(str, pilchard.run())), ["dog-1-2.x86_64"])

    def test_glob_arch(self):
        q = hawkey.Query(self.sack)
        pilchard = q.filter(name="pilchard", version="1.2.4", release="1",
                            arch__glob="*6*")
        res = list(map(str, pilchard.run()))
        self.assertItemsEqual(res, ["pilchard-1.2.4-1.x86_64",
                              "pilchard-1.2.4-1.i686"])

    def test_obsoletes(self):
        q = hawkey.Query(self.sack).filter(name="penny")
        o = hawkey.Query(self.sack)
        self.assertRaises(hawkey.QueryException, o.filter, obsoletes__gt=q)

        o = hawkey.Query(self.sack).filter(obsoletes=q)
        self.assertLength(o, 1)
        self.assertEqual(str(o[0]), "fool-1-5.noarch")

    def test_requires_with_query(self):
        q = hawkey.Query(self.sack).filter(name="fool")
        o = hawkey.Query(self.sack).filter(requires=q)
        self.assertLength(o, 1)
        self.assertEqual(str(o[0]), "walrus-2-6.noarch")

    def test_requires_with_package_list(self):
        q = hawkey.Query(self.sack).filter(name="fool")
        o = hawkey.Query(self.sack).filter(requires=q.run())
        self.assertLength(o, 1)
        self.assertEqual(str(o[0]), "walrus-2-6.noarch")

    def test_subquery_evaluated(self):
        q = hawkey.Query(self.sack).filter(name="penny")
        self.assertFalse(q.evaluated)
        self.assertLength(q.run(), 1)
        self.assertTrue(q.evaluated)
        o = hawkey.Query(self.sack).filter(obsoletes=q)
        self.assertTrue(q.evaluated)
        self.assertLength(q.run(), 1)
        self.assertLength(o, 1)

class TestOddArch(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_test_repo("ppc", "ppc.repo")

    def test_latest(self):
        q = hawkey.Query(self.sack).filter(latest=True)
        self.assertEqual(len(q), 1)

        q = hawkey.Query(self.sack).filter(latest_per_arch=True)
        self.assertEqual(len(q), 1)


class TestAllArch(base.TestCase):

    def setUp(self):
        self.sack1 = base.TestSack(repo_dir=self.repo_dir, arch="ppc64")
        self.sack2 = base.TestSack(repo_dir=self.repo_dir, arch="x86_64")
        self.sack3 = base.TestSack(repo_dir=self.repo_dir, all_arch=True)
        self.sack1.load_test_repo("test_ppc", "ppc.repo")
        self.sack2.load_test_repo("test_ppc", "ppc.repo")
        self.sack3.load_test_repo("test_ppc", "ppc.repo")

    def test_provides_all_arch_query(self):
        ppc_pkgs = hawkey.Query(self.sack1)
        self.assertGreater(len(ppc_pkgs), 0)
        pkg1 = ppc_pkgs[0]

        query_ppc = hawkey.Query(self.sack1).filter(provides=pkg1.provides[0])
        query_x86 = hawkey.Query(self.sack2).filter(provides=pkg1.provides[0])
        query_all = hawkey.Query(self.sack3).filter(provides=pkg1.provides[0])

        self.assertEqual(len(query_ppc), 1)
        self.assertEqual(len(query_x86), 0)
        self.assertEqual(len(query_all), 1)


class TestQuerySubclass(base.TestCase):
    class CustomQuery(hawkey.Query):
        pass

    def test_instance(self):
        sack = base.TestSack(repo_dir=self.repo_dir)
        q = self.CustomQuery(sack)
        self.assertIsInstance(q, self.CustomQuery)
        q = q.filter(name="pepper")
        self.assertIsInstance(q, self.CustomQuery)

class TestQueryFilterAdvisory(base.TestCase):
    """Tests related to packages and updateinfo metadata."""

    def setUp(self):
        """Prepare the test fixture."""
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_repo(load_updateinfo=True)
        self.expected_pkgs = hawkey.Query(self.sack).filter(nevra=[
                                                "mystery-devel-19.67-1.noarch",
                                                "tour-4-6.noarch"]
                                                ).run()

    def test_advisory(self):
        pkgs = hawkey.Query(self.sack).filter(advisory="BEATLES-1967-1127").run()
        self.assertEqual(pkgs, self.expected_pkgs)

    def test_advisory_type(self):
        pkgs = hawkey.Query(self.sack).filter(advisory_type="security").run()
        self.assertEqual(pkgs, self.expected_pkgs)

    def test_advisory_bug(self):
        pkgs = hawkey.Query(self.sack).filter(advisory_bug="0#paul").run()
        self.assertEqual(pkgs, self.expected_pkgs)

    def test_advisory_cve(self):
        pkgs = hawkey.Query(self.sack).filter(advisory_cve="CVE-1967-BEATLES").run()
        self.assertEqual(pkgs, self.expected_pkgs)

class TestQuerySetOperations(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        self.q = hawkey.Query(self.sack)
        self.q1 = self.q.filter(version="4")
        self.q2 = self.q.filter(name__glob="p*")

    def test_difference(self):
        qi = self.q1.difference(self.q2)
        difference = [p for p in self.q1 if p not in self.q2]
        self.assertEqual(qi.run(), difference)

    def test_intersection(self):
        qi = self.q1.intersection(self.q2)
        intersection = self.q.filter(version="4", name__glob="p*")
        self.assertEqual(qi.run(), intersection.run())

    def test_union(self):
        qu = self.q1.union(self.q2)
        union = set(self.q1.run() + self.q2.run())
        self.assertEqual(set(qu), union)

    def test_zzz_queries_not_modified(self):
        self.assertEqual(len(self.q1), 5)
        self.assertEqual(len(self.q2), 5)



#
# Copyright (C) 2014 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

"""Tests of the _hawkey.Advisory class."""

from __future__ import absolute_import
from __future__ import unicode_literals
from . import base

import datetime
import hawkey
import itertools
import time

def find_advisory(sack, id_):
    """Find an advisory with given ID."""
    # The function is needed because advisories cannot be retrieved directly.
    advisories_iterable = itertools.chain(
        (pkg.get_advisories(hawkey.LT) for pkg in hawkey.Query(sack)),
        (pkg.get_advisories(hawkey.GT | hawkey.EQ) for pkg in hawkey.Query(sack)))
    for advisory in itertools.chain.from_iterable(advisories_iterable):
        if advisory.id == id_:
            return advisory

class Test(base.TestCase):
    """Test case consisting of one random advisory."""

    def setUp(self):
        """Prepare the test fixture."""
        sack = base.TestSack(repo_dir=self.repo_dir)
        sack.load_repo(load_updateinfo=True)
        self.advisory = find_advisory(sack, 'FEDORA-2008-9969')

    def test_description(self):
        self.assertEqual(self.advisory.description, 'An example update to the tour package.')

    def test_packages(self):
        filenames = [apkg.filename for apkg in self.advisory.packages]
        self.assertEqual(filenames, ['tour.noarch.rpm'])

    def test_id(self):
        self.assertEqual(self.advisory.id, 'FEDORA-2008-9969')

    def test_references(self):
        urls = [ref.url for ref in self.advisory.references]
        self.assertEqual(urls,
                         ['https://bugzilla.redhat.com/show_bug.cgi?id=472090',
                          'https://bugzilla.gnome.com/show_bug.cgi?id=472091'])

    def test_rights(self):
        self.assertIsNone(self.advisory.rights)

    def test_title(self):
        self.assertEqual(self.advisory.title, 'lvm2-2.02.39-7.fc10')

    def test_type(self):
        self.assertEqual(self.advisory.type, hawkey.ADVISORY_BUGFIX)

    def test_updated(self):
        self.assertEqual(self.advisory.updated,
                         datetime.datetime(2008, 12, 9, 11, 31, 26) -
                         datetime.timedelta(seconds=time.timezone))



#
# Copyright (C) 2012-2013 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import
from . import _hawkey_test

import hawkey
import os

EXPECT_SYSTEM_NSOLVABLES = _hawkey_test.EXPECT_SYSTEM_NSOLVABLES
EXPECT_MAIN_NSOLVABLES = _hawkey_test.EXPECT_MAIN_NSOLVABLES
EXPECT_UPDATES_NSOLVABLES = _hawkey_test.EXPECT_UPDATES_NSOLVABLES
EXPECT_YUM_NSOLVABLES = _hawkey_test.EXPECT_YUM_NSOLVABLES
FIXED_ARCH = _hawkey_test.FIXED_ARCH
UNITTEST_DIR = _hawkey_test.UNITTEST_DIR
YUM_DIR_SUFFIX = _hawkey_test.YUM_DIR_SUFFIX

glob_for_repofiles = _hawkey_test.glob_for_repofiles

class TestSackMixin(object):
    def __init__(self, repo_dir):
        self.repo_dir = repo_dir

    def load_test_repo(self, name, fn, system=False):
        path = os.path.join(self.repo_dir, fn)
        _hawkey_test.load_repo(self, name, path, system)

    def load_system_repo(self, *args, **kwargs):
        path = os.path.join(self.repo_dir, "@System.repo")
        _hawkey_test.load_repo(self, hawkey.SYSTEM_REPO_NAME, path, True)



#
# Copyright (C) 2012-2019 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import

import copy
import os
import shutil
import sys
import tempfile
import unittest

from . import base
import libdnf
import hawkey
import hawkey.test

class TestSackTest(base.TestCase):
    def test_sanity(self):
        assert(self.repo_dir)
        sack = base.TestSack(repo_dir=self.repo_dir)
        self.assertEqual(len(sack), 0)

    def test_load_rpm(self):
        sack = base.TestSack(repo_dir=self.repo_dir)
        sack.load_system_repo()
        self.assertEqual(len(sack), hawkey.test.EXPECT_SYSTEM_NSOLVABLES)

    def test_load_yum(self):
        sack = base.TestSack(repo_dir=self.repo_dir)
        sack.load_system_repo()
        sack.load_repo(build_cache=True)
        self.assertEqual(len(sack), hawkey.test.EXPECT_YUM_NSOLVABLES +
                         hawkey.test.EXPECT_SYSTEM_NSOLVABLES)

    # Loading test using hawkey.Repo
    def test_load_yum_hawkey_Repo(self):
        sack = base.TestSack(repo_dir=self.repo_dir)
        sack.load_system_repo()
        sack.load_repo_hawkey_Repo(build_cache=True)
        self.assertEqual(len(sack), hawkey.test.EXPECT_YUM_NSOLVABLES +
                         hawkey.test.EXPECT_SYSTEM_NSOLVABLES)

    def test_cache_dir(self):
        sack = base.TestSack(repo_dir=self.repo_dir)
        self.assertTrue(sack.cache_dir.startswith("/tmp/pyhawkey"))

class BasicTest(unittest.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp(prefix="libdnf_test_")
        self.cache_dir = os.path.join(self.temp_dir, "cache")

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_creation(self):
        hawkey.Sack(arch="noarch")
        hawkey.Sack(arch="x86_64")

    def test_deepcopy(self):
        sack = hawkey.Sack()
        self.assertRaises(NotImplementedError, copy.deepcopy, sack)

    def test_creation_dir(self):
        sack = hawkey.Sack(cachedir=self.cache_dir)
        self.assertFalse(os.access(sack.cache_dir, os.F_OK))
        sack = hawkey.Sack(make_cache_dir=True)
        self.assertTrue(os.access(sack.cache_dir, os.F_OK))
        self.assertRaises(IOError, hawkey.Sack, "", make_cache_dir=True)

    def test_failed_load(self):
        sack = hawkey.Sack(cachedir=base.cachedir)
        self._conf = libdnf.conf.ConfigMain()
        repo_conf = libdnf.conf.ConfigRepo(self._conf)
        repo_conf.this.disown()  # _repo will be the owner of _config
        repo = libdnf.repo.Repo("name", repo_conf)
        self.assertRaises(IOError, sack.load_repo, repo)
        sack = hawkey.Sack()

    # Loading test using hawkey.Repo
    def test_failed_load_hawkey_Repo(self):
        sack = hawkey.Sack(cachedir=base.cachedir)
        repo = hawkey.Repo("name")
        self.assertRaises(IOError, sack.load_repo, repo)
        sack = hawkey.Sack()

    def test_unicoded_cachedir(self):
        # does not raise UnicodeEncodeError
        hawkey.Sack(cachedir=u"unicod\xe9")

    def test_evr_cmp(self):
        sack = hawkey.Sack()
        self.assertEqual(sack.evr_cmp("3:3.10-4", "3:3.10-4"), 0)
        self.assertLess(sack.evr_cmp("3.10-4", "3.10-5"), 0)
        self.assertGreater(sack.evr_cmp("3.11-4", "3.10-5"), 0)
        self.assertGreater(sack.evr_cmp("1:3.10-4", "3.10-5"), 0)

    def test_all_arch(self):
        sack = hawkey.Sack(arch="x86_64")
        # greater than noarch as it has picked up the default architecture
        self.assertGreater(len(sack.list_arches()), 1)
        sack2 = hawkey.Sack(all_arch=True)
        self.assertEqual(len(sack2.list_arches()), 0)


class PackageWrappingTest(base.TestCase):
    class MyPackage(hawkey.Package):
        def __init__(self, initobject, myval):
            super(PackageWrappingTest.MyPackage, self).__init__(initobject)
            self.initobject = initobject
            self.myval = myval

    def test_wrapping(self):
        sack = base.TestSack(repo_dir=self.repo_dir,
                                    PackageClass=self.MyPackage,
                                    package_userdata=42)
        sack.load_system_repo()
        pkg = sack.create_package(2)
        # it is the correct type:
        self.assertIsInstance(pkg, self.MyPackage)
        self.assertIsInstance(pkg, hawkey.Package)
        # it received userdata:
        self.assertEqual(pkg.myval, 42)
        # the common attributes are working:
        self.assertEqual(pkg.name, "baby")



#
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# Licensed under the GNU Lesser General Public License Version 2.1
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from __future__ import absolute_import
from . import base
from sys import version_info as python_version

import hawkey

class AdvisoriesTest(base.TestCase):
    """Tests related to packages and updateinfo metadata."""

    def setUp(self):
        """Prepare the test fixture."""
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_repo(load_updateinfo=True)

    def test(self):
        pkg = hawkey.Query(self.sack).filter(name='tour')[0]
        advisories = pkg.get_advisories(hawkey.GT)
        self.assertEqual(len(advisories), 1)
        self.assertEqual(advisories[0].id, u'FEDORA-2008-9969')

    def test_noadvisory(self):
        pkg = hawkey.Query(self.sack).filter(name='mystery-devel')[0]
        advisories = pkg.get_advisories(hawkey.GT)
        self.assertEqual(len(advisories), 0)

class PackageTest(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        q = hawkey.Query(self.sack).filter(name__eq="flying")
        self.pkg = list(q)[0]

    def test_instance(self):
        self.assertTrue(isinstance(self.pkg, hawkey.Package))

    def test_repr(self):
        regexp = r"<hawkey.Package object id \d+, flying-2-9\.noarch, @System>"
        self.assertRegex(repr(self.pkg), regexp)

    def test_str(self):
        self.assertEqual(str(self.pkg),
                         "flying-2-9.noarch")

    def test_hashability(self):
        """ Test that hawkey.Package objects are hashable.

            So they can be keys in a dict.
        """
        d = {}
        d[self.pkg] = True
        pkg2 = hawkey.Query(self.sack).filter(name__eq="fool").run()[0]
        d[pkg2] = 1
        self.assertEqual(len(d), 2)

        # if we get the same package twice it is a different object (hawkey does
        # not pool Packages at this time). It however compares equal.
        pkg3 = hawkey.Query(self.sack).filter(name__eq="fool").run()[0]
        self.assertIsNot(pkg2, pkg3)
        self.assertEqual(pkg2, pkg3)
        d[pkg3] += 1
        self.assertEqual(d[pkg2], 2)
        self.assertEqual(len(d), 2)

    def test_conflicts(self):
        pkg = base.by_name(self.sack, 'dog')
        self.assertItemsEqual(list(map(str, pkg.conflicts)), ('custard = 1.1',))

    def test_files(self):
        pkg = base.by_name(self.sack, 'fool')
        self.assertSequenceEqual(pkg.files, ('/no/answers',))

class PackageCmpTest(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_system_repo()
        self.sack.load_test_repo("main", "main.repo")
        self.pkg1 = base.by_name_repo(self.sack, "fool", hawkey.SYSTEM_REPO_NAME)

    def test_cmp(self):
        pkg2 = base.by_name_repo(self.sack, "fool", "main")
        # if nevra matches the packages are equal:
        self.assertEqual(self.pkg1, pkg2)

        # if the name doesn't match they are not equal:
        pkg2 = base.by_name_repo(self.sack, "hello", "main")
        self.assertNotEqual(self.pkg1, pkg2)

        # if nevr matches, but not arch, they are not equal:
        pkg1 = hawkey.split_nevra("semolina-2-0.x86_64").to_query(self.sack)[0]
        pkg2 = hawkey.split_nevra("semolina-2-0.i686").to_query(self.sack)[0]
        self.assertNotEqual(pkg1, pkg2)
        # however evr_cmp compares only the evr part:
        self.assertEqual(pkg1.evr_cmp(pkg2), 0)

        pkg1 = hawkey.split_nevra("jay-6.0-0.x86_64").to_query(self.sack)[0]
        pkg2 = hawkey.split_nevra("jay-5.0-0.x86_64").to_query(self.sack)[0]
        self.assertLess(pkg2, pkg1)
        self.assertGreater(pkg1.evr_cmp(pkg2), 0)

    def test_cmp_fail(self):
        # should not throw TypeError
        self.assertNotEqual(self.pkg1, "hawkey-package")
        self.assertNotEqual(self.pkg1, self.sack)

        if python_version.major > 3:
            self.assertRaises(TypeError, lambda: self.pkg1 <= "hawkey-package")

    def test_cmp_lt(self):
        greater_package = base.by_name_repo(self.sack, "hello", "main")

        self.assertLess(self.pkg1, greater_package)

    def test_cmp_gt(self):
        less_package = base.by_name_repo(self.sack, "baby", "main")

        self.assertGreater(self.pkg1, less_package)

class ChecksumsTest(base.TestCase):
    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_repo()
        self.pkg = hawkey.Query(self.sack).filter(name="mystery-devel")[0]

    def test_checksum(self):
        (chksum_type, chksum) = self.pkg.chksum
        self.assertEqual(len(chksum), 32)
        if python_version.major < 3:
            self.assertEqual(chksum[0], b'\x2e')
            self.assertEqual(chksum[31], b'\xf5')
        else:
            self.assertEqual(chksum[0], 0x2e)
            self.assertEqual(chksum[31], 0xf5)
        self.assertEqual(chksum_type, hawkey.CHKSUM_SHA256)

    def test_checksum_fail(self):
        sack = base.TestSack(repo_dir=self.repo_dir)
        sack.load_system_repo()
        pkg = base.by_name(sack, "fool")
        self.assertIsNone(pkg.chksum)
        self.assertIsNone(pkg.hdr_chksum)

    def test_sizes(self):
        pkg = base.by_name(self.sack, "mystery-devel")
        self.assertEqual(pkg.installsize, 59)
        self.assertEqual(pkg.downloadsize, 2329)

class FullPropertiesTest(base.TestCase):
    """Tests properties as seen in a real primary.xml."""

    def setUp(self):
        self.sack = base.TestSack(repo_dir=self.repo_dir)
        self.sack.load_repo()
        self.pkg = hawkey.Query(self.sack).filter(name="mystery-devel")[0]

    def test_baseurl(self):
        self.assertEqual(self.pkg.baseurl, 'disagree')

    def test_hdr_end(self):
        self.assertEqual(self.pkg.hdr_end, 2081)

    def test_sourcerpm(self):
        self.assertEqual(self.pkg.sourcerpm, 'mystery-19.67-1.src.rpm')
